#ifndef LINT
static char *rcsid="$Id: rtconfig.c,v 1.16 2003/09/20 12:02:32 crosser Exp $";
#endif
                                                                                
/*
	$Log: rtconfig.c,v $
	Revision 1.16  2003/09/20 12:02:32  crosser
	one more spam sample
	
	Revision 1.15  2003/09/14 22:26:32  crosser
	be verbose about loading shared objects
	
	Revision 1.14  2003/09/14 22:07:20  crosser
	clear first position if necessary
	
	Revision 1.13  2003/09/14 11:31:43  crosser
	code cleanup to silence -Wall warnings.  Actually, some of them where bugs.
	Still cannot figure how to initialize `union' without warning
	
	Revision 1.12  2003/09/13 18:35:03  crosser
	report config errors to syslog (because if we are respawning there
	is not stderr).  Compile-time warnings.
	
	Revision 1.11  2003/09/11 13:56:26  crosser
	check_ct - watch for empty body.  Test some broken cases
	
	Revision 1.10  2003/09/05 15:25:11  crosser
	ceck_ct plugin finally works.
	
	Revision 1.9  2003/09/05 10:10:42  crosser
	make config parameter "gcfg.modconfdir" and access method
	
	Revision 1.8  2003/09/03 16:07:43  crosser
	make pid file
	make zmsctl look and work without termios
	make zmsctl use socket name from config
	
	Revision 1.7  2003/09/02 22:20:11  crosser
	make initial stage run time configurable
	
	Revision 1.6  2003/09/02 21:58:06  crosser
	fix: last body part was not processed
	
	Revision 1.5  2003/09/02 08:05:01  crosser
	make dynamic loading work on Solaris
	
	Revision 1.4  2003/09/01 18:20:45  crosser
	proper test format
	debugging readconfig error - int var
	
	Revision 1.3  2003/09/01 16:39:22  crosser
	make DSO actually work
	
	Revision 1.2  2003/09/01 14:04:40  crosser
	make runtime config
	
	Revision 1.1  2003/09/01 09:39:21  crosser
	add head comments where necessary
	add run time config function
	
*/

/*
	WHAT IS IT:
		modularized contentfilter for Zmailer
	COPYRIGHT:
		(c) 2003 Eugene G. Crosser <crosser@average.org>
	LICENSE:
		The same set as apply to Zmailer code
*/

#include "config.h"

#ifdef STDC_HEADERS
# include <stdio.h>
# include <stdlib.h>
# include <string.h>
#endif
#ifdef HAVE_DLFCN_H
# include <dlfcn.h>
#endif
#ifdef HAVE_LINK_H
# include <link.h>
#endif

#include "report.h"
#include "rtconfig.h"

#define MAXMSGSIZE 8196
#define DEFAULT_SOCKPATH "/tmp/zmscanner"
#define DEFAULT_INITSTAGE "zmsg";
#define DEFAULT_FINALSTAGE "zfin";

int
verb_module(char *verb,char *data,char *fn,int count);

static int doload;
static char *moddir=DEFAULT_MODDIR;

static struct _verbs {
	char *verb;
	enum {badverb,integer,string,exec} vtype;
	union _w {
			int *i;
			char **s;
			int (*e)(char *v,char *p,char *fn,int c);
		} where;
} verbs[] = {
	{"maxsize",	integer,	{&gcfg.maxsize}},
	{"socket",	string,		{&gcfg.socket}},
	{"initstage",	string,		{&gcfg.initstage}},
	{"finalstage",	string,		{&gcfg.finalstage}},
	{"pidfile",	string,		{&gcfg.pidfile}},
	{"modconfdir",	string,		{&gcfg.modconfdir}},
	{"moddir",	string,		{&moddir}},
	{"module",	exec,		{verb_module}},
	{NULL,		badverb,	{NULL}}
};

int
readconfig(char *fn,int loadmodules)
{
	FILE *fp;
	char buf[256];
	int count=0;

	doload=loadmodules;

	gcfg.socket=DEFAULT_SOCKPATH;
	gcfg.initstage=DEFAULT_INITSTAGE;
	gcfg.finalstage=DEFAULT_FINALSTAGE;
	gcfg.pidfile=NULL;
	gcfg.maxsize=MAXMSGSIZE;

	DPRINT(("readconfig fn=%s\n",fn));
	if ((fp=fopen(fn,"r")) == NULL) {
		perror(fn);
		return -1;
	}
	while (fgets(buf,sizeof(buf)-1,fp)) {
		char *p,*q;
		int i,factor;

		count++;
#if 0
		DPRINT(("readconfig line %d: \"%s\"\n",count,buf));
#endif
		if (buf[0] == '#') continue;
		buf[sizeof(buf)-1]='\0';
		p=buf+strlen(buf)-1;
		while ((p >= buf) && ((*p == '\r') || (*p == '\n') ||
				     (*p == ' ') || (*p == '\t'))) *p--='\0';
		if (buf[0] == '\0') continue;
		for (p=buf;(*p != ' ') && (*p != '\t') && (*p != '\0');p++) ;
		*p++='\0';
		while ((*p == ' ') || (*p == '\t')) p++;
		for (i=0;verbs[i].verb;i++) {
			if (strcmp(verbs[i].verb,buf) == 0) break;
		}
		switch (verbs[i].vtype) {
		case integer:
			q=p+strlen(p)-1;
			switch (*q) {
			case 'k':
			case 'K':
				factor=1024;
				*q='\0';
				break;
			case 'm':
			case 'M':
				factor=1024*1024;
				*q='\0';
				break;
			default:
				factor=1;
				break;
			}
			*verbs[i].where.i=atoi(p)*factor;
			break;
		case string:
			*verbs[i].where.s=(char*)malloc(strlen(p)+1);
			strcpy(*verbs[i].where.s,p);
			break;
		case exec:
			(void)(*verbs[i].where.e)(buf,p,fn,count);
			break;
		case badverb:
			ERRLOG((LOG_ERR,"%s(%d): bad verb \"%s\"\n",
					fn,count,buf));
			break;
		}
	}
	fclose(fp);
	DPRINT(("moddir=%s sock=%s maxsize=%d\n",
				moddir,gcfg.socket,gcfg.maxsize));
	return 0;
}

int
verb_module(char *verb,char *data,char *fn,int count)
{
#ifdef HAVE_DLOPEN
	void *dso;
	char *dlerr;
	char buf[256];
	void (*mod_init_func)(void);

	if (!doload) return 0;

	DPRINT(("loading module \"%s\"\n",data));
	snprintf(buf,sizeof(buf)-1,"%s/%s",moddir,data);
	dso=dlopen(buf,RTLD_NOW);
	if ((dlerr=dlerror()) != NULL)  {
		ERRLOG((LOG_ERR,"%s(%d): %s\n",fn,count,dlerr));
		return 1;
	}
	mod_init_func=dlsym(dso,"zms_mod_init");
	if ((dlerr=dlerror()) != NULL)  {
		ERRLOG((LOG_ERR,"%s(%d): %s\n",fn,count,dlerr));
		return 1;
	}
	ERRLOG((LOG_INFO,"loading %s",buf));
	(*mod_init_func)();
	return 0;
#else /* HAVE_DLOPEN */
	ERRLOG((LOG_ERR,"%s(%d): %s not supported without dlopen()\n",
			fn,count,verb));
	return 1;
#endif /* HAVE_DLOPEN */
}
