#ifndef LINT
static char *rcsid="$Id: breakbody.c,v 1.11 2004/06/13 10:21:00 crosser Exp $";
#endif

/*
	$Log: breakbody.c,v $
	Revision 1.11  2004/06/13 10:21:00  crosser
	clean up pthread check
	
	Revision 1.10  2003/09/14 21:16:18  crosser
	I hardly beleive it but the new varpool thing seems to work!
	
	Revision 1.9  2003/09/12 08:45:24  crosser
	save queue file if cannot parse multipart
	
	Revision 1.8  2003/09/11 08:59:29  crosser
	name is LOG_WARNING
	
	Revision 1.7  2003/09/11 08:58:06  crosser
	make syslog WARN where things not under our control
	
	Revision 1.6  2003/09/11 08:56:56  crosser
	cosmetic and cleanup
	
	Revision 1.5  2003/09/08 12:55:55  crosser
	make static and dynamic plugins syntactically identical
	
	Revision 1.4  2003/09/06 20:27:45  crosser
	nicely allow broken end-of-message boundary
	
	Revision 1.3  2003/09/04 19:36:07  crosser
	fix boundaries assignment; thing works.
	
	Revision 1.2  2003/09/04 18:03:41  crosser
	changed (and broke) scanning for attributes in the headers
	
	Revision 1.1  2003/09/03 21:09:22  crosser
	completed switch from (start,len) pairs to slab_t descriptors
	pass opaque attribute block between stages - so could separate
	breakrfc822 into two stages (multibody is separate now)
	
	Revision 1.6  2003/09/03 18:32:50  crosser
	work in progress on new calling conventions
	
	Revision 1.5  2003/09/02 22:06:05  crosser
	make mattr_t visible to plugins
	define slab_t in preparation to change of calling conventions
	
	Revision 1.4  2003/09/02 21:58:06  crosser
	fix: last body part was not processed
	
	Revision 1.3  2003/09/02 12:56:01  crosser
	implement parsing of multipart messages
	
	Revision 1.2  2003/09/02 09:59:34  crosser
	fix missing/extra headers and off by one errors
	
	Revision 1.1  2003/09/01 22:39:00  crosser
	scan interesting headers
	FIXME: skip processing of last header
	
	
*/
                                                                                
/*
	WHAT IS IT:
		modularized contentfilter for Zmailer
	COPYRIGHT:
		(c) 2003 Eugene G. Crosser <crosser@average.org>
	LICENSE:
		The same set as apply to Zmailer code
*/

#include "config.h"

#include <sys/types.h>
#ifdef STDC_HEADERS
# include <stdio.h>
# include <string.h>
# include <ctype.h>
#endif

#include "report.h"
#include "zmscanner.h"

static int
breakbody_setup(void **cfgp)
{
	DPRINT(("breakbody_setup called\n"));
	(*cfgp)=NULL;
	return 0;
}

static void
breakbody_term(void **cfgp)
{
	DPRINT(("breakbody_term called\n"));
}

static int
breakbody_scan(char *stage,int depth,slab_t data,varpool_t vp,void *priv)
{
	int rc;
	slab_t edata;
	slab_t bndry;
	char *p,*bp,*sop,*eop;
	enum {
		waitnl,waitminus1,waitminus2,inboundary,
		endbound,waitnextnl,waitlastminus,trail
	     } state;

	DPRINT(("breakbody_scan called size=%d\n",slab_size(data)));
	if (data.atr == NULL) {
		ERRLOG((LOG_WARNING,"breakbody called but atr not supplied"));
		return ZMSCAN_CONTINUE;
	}
	bndry=((mattr_t*)data.atr)->boundary;
	if (slab_size(bndry) == 0) {
		ERRLOG((LOG_WARNING,"breakbody called with empty boundary"));
		saveqfile();
		return ZMSCAN_CONTINUE;
	}

	DPRINT(("boundary is \"%.*s\"\n",slab_size(bndry),bndry.beg));

	memset(&edata,0,sizeof(edata));
	sop=data.end;
	eop=data.beg;
	state=waitnl;
	for (p=data.beg;p<data.end;p++) {
		if (*p == '\r') continue;
#if 0
		DPRINT(("switched from %d",state));
#endif
		switch (state) {
		case waitnl:
			if (*p == '\n') {
				eop=p;
				state=waitminus1;
			}
			break;
		case waitminus1:
			if (*p == '-') {
				state=waitminus2;
			}
			else if (*p == '\n') {
				eop=p;
			} else state=waitnl;
			break;
		case waitminus2:
			if (*p == '-') {
				state=inboundary;
				bp=bndry.beg;
			} else state=waitnl;
			break;
		case inboundary:
			if (bp >= bndry.end) {
				if (eop > sop) {
					edata.beg=sop;
					edata.end=eop;
					rc=scandata("rfc822",depth+1,edata,vp);
					if (rc != ZMSCAN_CONTINUE) return rc;
					sop=data.end; /* no rescan on errors */
				}
				if (*p == '\n') {
					state=endbound;
				} else if (*p == '-') {
					state=waitlastminus;
				} else {
					/* actually this is the error */
					DPRINT(("\nbad char '%02x'\n\t",*p));
					state=waitnextnl;
				}
			}
			else if (*bp == *p) bp++;
			else state=waitnl;
			break;
		case endbound:
			sop=p;
			state=waitnl;
			break;
		case waitnextnl:
			/* this is a format error, actually... */
			if (*p == '\n') {
				state=endbound;
			}
			break;
		case waitlastminus:
			if (*p == '-') state=trail;
			else state=waitnl; /* try to find more data... */
			break;
		case trail:
			break; /* skip till the end */
		}
#if 0
		DPRINT((" to %d looking at \"%.20s\"\n",state,p));
#endif
	}
	if (state != trail) { /* process incomplete data */
		edata.beg=sop;
		edata.end=eop;
		rc=scandata("rfc822",depth+1,edata,vp);
		if (rc != ZMSCAN_CONTINUE) return rc;
	}
	return ZMSCAN_CONTINUE;
}

ZMS_MODULE("multibody","breakbody",breakbody_setup,
				breakbody_term,breakbody_scan);
