#ifndef LINT
static char *rcsid="$Id: breakrfc822.c,v 1.18 2003/09/14 21:16:18 crosser Exp $";
#endif

/*
	$Log: breakrfc822.c,v $
	Revision 1.18  2003/09/14 21:16:18  crosser
	I hardly beleive it but the new varpool thing seems to work!
	
	Revision 1.17  2003/09/13 11:41:16  crosser
	make new header parsing actually work
	
	Revision 1.16  2003/09/12 15:22:41  crosser
	break parsing to separate modules (just placeholders for now)
	
	Revision 1.15  2003/09/12 09:05:07  crosser
	better check for attributes (still not completely correct)
	
	Revision 1.14  2003/09/11 12:38:44  crosser
	redesign checkhdr
	
	Revision 1.13  2003/09/09 20:02:10  crosser
	complete dispatcher; change "body" stage to "content" for consistency
	
	Revision 1.12  2003/09/09 17:40:29  crosser
	wip on dispatcher
	
	Revision 1.11  2003/09/09 08:19:15  crosser
	forgot "default" label
	
	Revision 1.10  2003/09/08 12:55:55  crosser
	make static and dynamic plugins syntactically identical
	
	Revision 1.9  2003/09/04 19:36:07  crosser
	fix boundaries assignment; thing works.
	
	Revision 1.8  2003/09/04 18:03:41  crosser
	changed (and broke) scanning for attributes in the headers
	
	Revision 1.7  2003/09/03 21:09:22  crosser
	completed switch from (start,len) pairs to slab_t descriptors
	pass opaque attribute block between stages - so could separate
	breakrfc822 into two stages (multibody is separate now)
	
	Revision 1.6  2003/09/03 18:32:50  crosser
	work in progress on new calling conventions
	
	Revision 1.5  2003/09/02 22:06:05  crosser
	make mattr_t visible to plugins
	define slab_t in preparation to change of calling conventions
	
	Revision 1.4  2003/09/02 21:58:06  crosser
	fix: last body part was not processed
	
	Revision 1.3  2003/09/02 12:56:01  crosser
	implement parsing of multipart messages
	
	Revision 1.2  2003/09/02 09:59:34  crosser
	fix missing/extra headers and off by one errors
	
	Revision 1.1  2003/09/01 22:39:00  crosser
	scan interesting headers
	FIXME: skip processing of last header
	
	
*/
                                                                                
/*
	WHAT IS IT:
		modularized contentfilter for Zmailer
	COPYRIGHT:
		(c) 2003 Eugene G. Crosser <crosser@average.org>
	LICENSE:
		The same set as apply to Zmailer code
*/

#include "config.h"

#include <sys/types.h>
#ifdef STDC_HEADERS
# include <stdio.h>
# include <string.h>
# include <ctype.h>
#endif

#include "report.h"
#include "zmscanner.h"

static int
breakrfc822_setup(void **cfgp)
{
	DPRINT(("breakrfc822_setup called\n"));
	(*cfgp)=NULL;
	return 0;
}

static void
breakrfc822_term(void **cfgp)
{
	DPRINT(("breakrfc822_term called\n"));
}

static int
breakrfc822_scan(char *stage,int depth,slab_t data,varpool_t vp,void *priv)
{
	int rc;
	char *p,*hs,*bs;
	int nlseen;
	mattr_t matr;
	slab_t edata;

	DPRINT(("breakrfc822_scan called size=%d\n",slab_size(data)));

	/* in case that there is no content-type/content-encoding in
	   the message, fill in attribute block with default values */
	memset(&matr,0,sizeof(matr));
	matr.btype=t_single;
	matr.benc=e_plain;

	memset(&edata,0,sizeof(edata));
	edata.atr=(void *)&matr;

	hs=data.beg;
	nlseen=0;
	for (p=data.beg;p<data.end;p++) {
		if (*p == '\r') continue;
		if (nlseen) { /* first character after newline */
			if ((*p != ' ') && (*p != '\t')) {
				DPRINT(("header at offset %d\n",
							hs-data.beg));
				edata.beg=hs;
				edata.end=p;
				rc=scandata("rfc822hdr",depth,edata,vp);
				if (rc != ZMSCAN_CONTINUE) return rc;
				hs=p;
			}
			nlseen=0; /* should we tolerate sep line with space? */
			if (*p == '\n') break; /* second newline in a row */
		}
		if (*p == '\n') nlseen=1;
	}

	/* matr should have been filled as a side effect of header scanning */
	DPRINT(("breakrfc822_scan atr btype=%d\n",matr.btype));
	DPRINT(("breakrfc822_scan atr benc=%d\n",matr.benc));
	DPRINT(("breakrfc822_scan atr content_type=\"%.*s\"\n",
			slab_size(matr.content_type),matr.content_type.beg));
	DPRINT(("breakrfc822_scan atr boundary=\"%.*s\"\n",
			slab_size(matr.boundary),matr.boundary.beg));
	DPRINT(("breakrfc822_scan atr name=\"%.*s\"\n",
			slab_size(matr.name),matr.name.beg));
	DPRINT(("breakrfc822_scan atr charset=\"%.*s\"\n",
			slab_size(matr.charset), matr.charset.beg));
	if ((matr.btype == t_multipart) && (slab_size(matr.boundary) == 0)) {
		/* I've seen DSNs from some broken MTA that generated
		   mulipart/mixed entities without boundary! Being
		   obvously broken, ew have a choice of treating such
		   things as either text/plain or message/rfc822.
		   In my case, body of the entity was in fact an
		   enclosed message.  So, let's try this: */
		matr.btype=t_message;
	}

	if (p < data.end) p++;
	DPRINT(("body starts at %d: \"%.20s\"\n",p-data.beg,p));
	edata.beg=p;
	edata.end=data.end;
	switch (matr.btype) {
	case t_single:
		switch (matr.benc) {
		case e_plain:
			return scandata("content",depth,edata,vp);
			break;
		case e_qp:
			return scandata("body_qp",depth,edata,vp);
			break;
		case e_base64:
			return scandata("body_base64",depth,edata,vp);
			break;
		case e_uue:
			return scandata("body_uue",depth,edata,vp);
			break;
		default:
			ERRLOG((LOG_ERR,"impossible message encoding"));
			return ZMSCAN_CONTINUE;
		}
		break;
	case t_message:
		return scandata("rfc822",depth+1,edata,vp);
		break;
	case t_multipart:
		return scandata("multibody",depth,edata,vp);
		break;
	default:
		ERRLOG((LOG_ERR,"impossible message type"));
		return ZMSCAN_CONTINUE;
	}
}

ZMS_MODULE("rfc822","breakrfc822",breakrfc822_setup,
				breakrfc822_term,breakrfc822_scan);
